<?php

namespace Drupal\entity_reference_preview\Form;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\entity_reference_preview\Plugin\PreviewDetector\StatePreviewDetector;
use Drupal\entity_reference_preview\PreviewDetectorPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Entity Reference Preview form.
 */
final class PreviewActionsForm extends FormBase {

  /**
   * The state detector plugin.
   *
   * @var \Drupal\entity_reference_preview\Plugin\PreviewDetector\StatePreviewDetector
   */
  private $stateDetector;

  /**
   * PreviewActionsForm constructor.
   *
   * @param \Drupal\entity_reference_preview\Plugin\PreviewDetector\StatePreviewDetector $state_detector
   *   The state detector plugin.
   */
  public function __construct(StatePreviewDetector $state_detector) {
    $this->stateDetector = $state_detector;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $state_detector = $container->get(PreviewDetectorPluginManager::class)
      ->createInstance('state');
    return new static($state_detector);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_reference_preview_preview_actions';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $is_previewing = $form_state->getValue('is_previewing', FALSE);
    $previewing_state = $form_state->getValue('previewing_via_state', FALSE);
    $current_status = $previewing_state ? 'previewing' : 'stopped';
    $form['current'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#attributes' => ['class' => ['description']],
      '#value' => $this->getFeedbackMessage($previewing_state),
    ];
    $form['controls'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['form-controls']],
      '#access' => !$is_previewing || $previewing_state,
    ];
    $form['controls']['status'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['preview--' . $current_status]],
      'value' => [
        '#prefix' => '<span class="circle"></span>',
        '#markup' => $this->t('Current status is: %status.', [
          '%status' => $current_status,
        ]),
      ],
    ];
    $form['controls']['actions'] = [
      '#type' => 'actions',
    ];
    $form['controls']['actions']['start'] = [
      '#type' => 'submit',
      '#name' => 'start',
      '#access' => !$previewing_state,
      '#value' => $this->t('Start'),
    ];
    $form['controls']['actions']['stop'] = [
      '#type' => 'submit',
      '#name' => 'stop',
      '#access' => $previewing_state,
      '#value' => $this->t('Stop'),
    ];
    $form['controls']['#attached'] = [
      'library' => ['entity_reference_preview/preview_controls'],
    ];
    $form['actions_info'] = [
      '#type' => 'html_tag',
      '#tag' => 'small',
      '#value' => $this->t('Unable to show controls. Currently previewing via another method.'),
      '#access' => $is_previewing && !$previewing_state,
    ];

    (new CacheableMetadata())
      ->setCacheContexts(['entity_reference_preview'])
      ->applyTo($form);
    return $form;
  }

  /**
   * Get the feedback message for the preview status.
   *
   * @param bool $is_previewing
   *   TRUE indicates if the request is for preview.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The markup.
   */
  private function getFeedbackMessage(bool $is_previewing): TranslatableMarkup {
    $url = Url::fromRoute('entity_reference_preview.settings')->toString();
    return $is_previewing
      ? $this->t('All the entity references in this page using the formatter with preview are showing the latest version instead of the publicly available version.')
      : $this->t(
        'Enable reference preview to see the latest versions of the embedded referenced entities. When <a class="inline-link" href="@url">enabled</a>, users with access to the preview indicator will see a mark on embedded entities that have a draft available for preview.',
        ['@url' => $url]
      );
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $action = $form_state->getTriggeringElement()['#name'];
    if (!in_array($action, ['start', 'stop'])) {
      $form_state->setError($form, $this->t('Invalid action.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $action = $form_state->getTriggeringElement()['#name'];
    // Set the correct state value.
    $action === 'start'
      ? $this->stateDetector->start()
      : $this->stateDetector->stop();
    // Give feedback to the user.
    $message = $action === 'start'
      ? $this->t('Now previewing all referenced entities.')
      : $this->t('Previewing has been stopped. Reverted to normal entity reference behavior.');
    $this->messenger()->addStatus($message);
  }

}
